// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.usi.ch/carzaniga/siena/
//
//  Authors: Antonio Carzaniga
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2005 Antonio Carzaniga
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to one of the authors.
//
//
// $Id: bdd_util.h,v 1.3 2010-03-11 15:16:38 carzanig Exp $
//
#ifndef BDD_UTIL_H
#define BDD_UTIL_H

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <cstdio> // needed by cudd
#include <cudd.h>

namespace siena {

    struct BDDNode {
	int pos;
	BDDNode * true_link;
	BDDNode * false_link;

	BDDNode(): pos(-1), true_link(0), false_link(0) {};
	BDDNode(int p): pos(p), true_link(0), false_link(0) {};
	BDDNode(int i, BDDNode * t, BDDNode * f)
	    : pos(i), true_link(t), false_link(f) {};
    };

    struct CBQueue {
	CBQueue * next;
	const DdNode * cudd_node;
	BDDNode * n;

	CBQueue(CBQueue * nx, const DdNode * c, BDDNode * b)
	    : next(nx), cudd_node(c), n(b) {};
    };

    typedef std::map<const DdNode *, BDDNode *> nodemap_t;
} // end namespace siena

#endif
