// -*- C++ -*-
//
//  This file is part of Siena, a wide-area event notification system.
//  See http://www.inf.usi.ch/carzaniga/siena/
//
//  Authors: Antonio Carzaniga
//  See the file AUTHORS for full details. 
//
//  Copyright (C) 2003 University of Colorado
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
//  USA, or send email to one of the authors.
//
//
// $Id: alloca.cc,v 1.4 2010-03-11 15:16:45 carzanig Exp $
//
#include <iostream>

#include <siena/allocator.h>
#include "simple_fwd_types.h"

using std::cout;
using std::endl;
using namespace siena;

struct X {
    char c;
    
    X() : c('X') {};
};

struct Y {
    char c;
    int i;

    Y(int y) : c('Y'), i(y) {};
};

struct Z {
    char c;
    int i;
    void * p;

    Z(int z) : c('Z'), i(z), p(this) {};
};

int main(int argc, char *argv[]) {
    FTAllocator mem;
    X * x;
    Y * y;
    Z * z;
    simple_value * v;

    for(int i = 0; i < 100000; ++i) {
        x = new (mem) X;
        y = new (mem) Y(i);
        z = new (mem) Z(i);
        v = new (mem) simple_value(static_cast<siena::int_t>(i));
        if (x->c != 'X' || y->i != i || z->i != i || v->int_value() != i)
            return 1;
    }
    unsigned long es = 100000 * (sizeof(X)+sizeof(Y)+sizeof(Z)+sizeof(simple_value));
    if (mem.size() != es) {
	cout << "wrong mem size " << mem.size() 
	     << ". Expecting " << es << endl;
    } 
    mem.recycle();
    if (mem.size() != 0) {
	cout << "wrong mem size " << mem.size() << ". Expecting 0." << endl;
    } 
    for(int i = 0; i < 100000; ++i) {
        x = new (mem) X;
        y = new (mem) Y(i);
        z = new (mem) Z(i);
        v = new (mem) simple_value(static_cast<siena::int_t>(i));
        if (x->c != 'X' || y->i != i || z->i != i || v->int_value() != i)
            return 1;
    }
    if (mem.size() != es) {
	cout << "wrong mem size " << mem.size() 
	     << ". Expecting " << es << endl;
    } 
    mem.clear();
    if (mem.size() != 0) {
	cout << "wrong mem size " << mem.size() << ". Expecting 0." << endl;
    } 
    return 0;
}

